/*
  Copyright (c) 2008-2009, Dominik Reichl <dominik.reichl@t-online.de>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/

#include "TanUpgrade.h"
#include "PwUtilCp.h"

#include <string>
#include <assert.h>

#include <boost/lexical_cast.hpp>

static IKpAPI* g_pAPI = NULL;

#define LCL_MENU_ITEM_COUNT 1
static KP_MENU_ITEM g_menuItems[LCL_MENU_ITEM_COUNT];

KP_IMPL_STDREFIMPL(CTanUpgradeImpl)

BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	UNREFERENCED_PARAMETER(hinstDLL);
	UNREFERENCED_PARAMETER(fdwReason);
	UNREFERENCED_PARAMETER(lpvReserved);

	return TRUE;
}

KP_EXPORT HRESULT KP_API KP_I_INITIALIZELIB_DECL(IKpUnknown* pAPI)
{
	UNREFERENCED_PARAMETER(pAPI);
	return S_OK;
}

KP_EXPORT HRESULT KP_API KP_I_RELEASELIB_DECL(IKpUnknown* pAPI)
{
	UNREFERENCED_PARAMETER(pAPI);
	SAFE_RELEASE(g_pAPI);
	return S_OK;
}

KP_EXPORT HRESULT KP_API KP_I_CREATEINSTANCE_DECL(REFIID riid, void** ppvObject,
	IKpUnknown* pAPI)
{
	*ppvObject = NULL;

	if(pAPI == NULL) return E_UNEXPECTED; // Require API

	if((riid == IID_IKpUnknown) || (riid == IID_IKpPlugin))
	{
		if(pAPI->QueryInterface(IID_IKpAPI, (void**)&g_pAPI) != S_OK)
			return E_FAIL;

		IKpPlugin* pImpl = new CTanUpgradeImpl();
		*ppvObject = ((riid == IID_IKpPlugin) ? pImpl : (IKpUnknown*)pImpl);
		return S_OK;
	}

	return E_NOINTERFACE;
}

CTanUpgradeImpl::CTanUpgradeImpl()
{
	KP_IMPL_CONSTRUCT;

	ZeroMemory(&g_menuItems, sizeof(KP_MENU_ITEM) * LCL_MENU_ITEM_COUNT);

	g_menuItems[0].lpCommandString = _T("Upgrade All TANs");
	g_menuItems[0].dwIcon = 11;
}

STDMETHODIMP CTanUpgradeImpl::QueryInterface(REFIID riid, void** ppvObject)
{
	KP_SUPPORT_INTERFACE(IID_IKpUnknown, IKpUnknown);
	KP_SUPPORT_INTERFACE(IID_IKpPlugin, IKpPlugin);

	*ppvObject = NULL;
	return E_NOINTERFACE;
}

STDMETHODIMP_(BOOL) CTanUpgradeImpl::OnMessage(DWORD dwCode, LPARAM lParamW,
	LPARAM lParamL)
{
	UNREFERENCED_PARAMETER(lParamW);
	UNREFERENCED_PARAMETER(lParamL);

	switch(dwCode)
	{
	case KPM_DIRECT_CONFIG:
		MessageBox(g_pAPI->GetMainWindowHandle(), _T("This plugin doesn't have any options."),
			KPP_PRODUCT_NAME, MB_OK | MB_ICONINFORMATION);
		break;

	case KPM_PLUGIN_INFO:
		MessageBox(g_pAPI->GetMainWindowHandle(), _T("Upgrade TANs of KeePass 1.10 and earlier for usage in KeePass 1.11 and higher."),
			KPP_PRODUCT_NAME, MB_OK | MB_ICONINFORMATION);
		break;

	case KPM_DIRECT_EXEC:
		My_ProcessDirectCall(lParamW, lParamL);
		break;

	case KPM_INIT_MENU_POPUP:
		if(g_pAPI->IsFileOpen() == FALSE)
			g_menuItems[0].dwFlags |= KPMIF_DISABLED; // Disable menu item
		else
			g_menuItems[0].dwFlags &= ~KPMIF_DISABLED; // Enable menu item
		break;

	default:
		break;
	}

	return TRUE;
}

STDMETHODIMP_(LPCTSTR) CTanUpgradeImpl::GetProperty(LPCTSTR lpName)
{
	UNREFERENCED_PARAMETER(lpName);
	return NULL;
}

STDMETHODIMP CTanUpgradeImpl::SetProperty(LPCTSTR lpName, LPCTSTR lpValue)
{
	UNREFERENCED_PARAMETER(lpName);
	UNREFERENCED_PARAMETER(lpValue);
	return E_FAIL;
}

STDMETHODIMP_(DWORD) CTanUpgradeImpl::GetMenuItemCount()
{
	return LCL_MENU_ITEM_COUNT;
}

STDMETHODIMP_(KP_MENU_ITEM*) CTanUpgradeImpl::GetMenuItems()
{
	return &g_menuItems[0];
}

void My_ProcessDirectCall(LPARAM lParamW, LPARAM lParamL)
{
	UNREFERENCED_PARAMETER(lParamL);

	if(lParamW != (LPARAM)g_menuItems[0].dwCommandID) return;
	if(g_pAPI->IsFileOpen() == FALSE) return;

	IKpDatabase* pMgr = NULL;
	if(g_pAPI->QueryInstance(SCLSID_KpDatabase, IID_IKpDatabase, (void**)&pMgr) != S_OK)
	{
		assert(false);
		return;
	}

	IKpUtilities* pUtil = NULL;
	if(g_pAPI->QueryInstance(SCLSID_KpUtilities, IID_IKpUtilities, (void**)&pUtil) != S_OK)
	{
		assert(false);
		return;
	}

	if((pMgr == NULL) || (pUtil == NULL)) { assert(false); return; }

	DWORD dwUpgraded = 0;
	for(DWORD dwEntry = 0; dwEntry < pMgr->GetEntryCount(); ++dwEntry)
	{
		PW_ENTRY* pe = pMgr->GetEntry(dwEntry);
		if(pe == NULL) { assert(false); continue; }

		if(pUtil->IsTANEntry(pe) == FALSE) continue;
		if(pe->pszUserName[0] != 0) continue;
		if(pe->pszURL[0] != 0) continue;

		if(IsTanIndex(pe->pszAdditional))
		{
			PW_ENTRY peNew;

			pMgr->UnlockEntryPassword(pe);
			if(Pu_MemAllocCopyEntry(pe, &peNew) == FALSE) { assert(false); }
			pMgr->LockEntryPassword(pe);

			TCHAR* lpIndex = peNew.pszAdditional;
			peNew.pszAdditional = peNew.pszUserName;
			peNew.pszUserName = lpIndex;

			if(pMgr->SetEntry(dwEntry, &peNew) == FALSE) { assert(false); }

			Pu_MemFreeEntry(&peNew);
			++dwUpgraded;
		}
	}

	HWND hWnd = g_pAPI->GetMainWindowHandle();
	if(dwUpgraded > 0)
	{
		g_pAPI->SetFileModified(TRUE);
		g_pAPI->UpdateEntryList();
		g_pAPI->UpdateToolBar();

		std::basic_string<TCHAR> str = _T("Successfully upgraded ");
		str += boost::lexical_cast<std::basic_string<TCHAR> >(dwUpgraded);
		str += _T(" TAN entries!");

		MessageBox(hWnd, str.c_str(), KPP_PRODUCT_NAME, MB_ICONINFORMATION | MB_OK);
	}
	else MessageBox(hWnd, _T("No upgradable TANs found!"), KPP_PRODUCT_NAME,
		MB_ICONINFORMATION | MB_OK);

	pMgr->Release();
	pUtil->Release();
}

bool IsTanIndex(LPCTSTR lpString)
{
	if((lpString == NULL) || (lpString[0] == 0)) return false;

	DWORD i = 0;
	while(lpString[i] != 0)
	{
		if((lpString[i] < _T('0')) || (lpString[i] > _T('9'))) return false;

		++i;
	}

	return true;
}
